﻿using UnityEngine;

namespace hry.labs.NPC.AI.Actions
{

    [CreateAssetMenu(fileName = "Wander", menuName = "Actions/Wander")]
    public class Wander : Action
    {

        public override void Act(PlanningAgent agent)
        {
            Vector3 direction = agent.target.variable - agent.transform.position;
            direction.Normalize();
            Vector3 offset = Random.onUnitSphere * agent.wanderRadius.variable + direction;
            // Position it down
            Vector3 point = direction * agent.wanderSphereDistance.variable + agent.transform.position + offset;
            point.y = agent.transform.position.y;
            Ray ray = new Ray
            {
                origin = point + Vector3.up * 1.0f,
                direction = Vector3.down
            };
            RaycastHit hit;
            if (Physics.Raycast(ray, out hit, 10.0f))
            {
                agent.target.variable = Vector3.Lerp(hit.point, agent.target.variable, 0.95f);
            }
        }

#if UNITY_EDITOR
        public override void DrawGizmo(PlanningAgent agent)
        {
            Gizmos.color = Color.gray;
            Vector3 direction = agent.target.variable - agent.transform.position;
            direction.Normalize();
            Vector3 offset = Random.onUnitSphere * agent.wanderRadius.variable + direction;
            Vector3 center = direction * agent.wanderSphereDistance.variable + agent.transform.position;
            // Position it down
            Vector3 point = center + offset;
            point.y = agent.transform.position.y;
            Ray ray = new Ray
            {
                origin = point + Vector3.up * 1.0f,
                direction = Vector3.down
            };
            RaycastHit hit;
            if (Physics.Raycast(ray, out hit, 10.0f))
            {
                agent.target.variable = Vector3.Lerp(hit.point, agent.target.variable, 0.95f);
            }

            Gizmos.DrawWireSphere(center, agent.wanderRadius.variable);
            Gizmos.DrawLine(center, agent.target.variable);
        }
#endif
    }

}