﻿using UnityEngine;

namespace hry.labs.NPC.AI.Steering
{

    [CreateAssetMenu(fileName ="CollisionAvoidance", menuName ="Steering/Collision Avoidance")]
    public class CollisionAvoidance : Steer
    {

        public override Vector3 Act(SteeringAgent agent)
        {
            Vector3 avoidanceForce = Vector3.zero;
            Vector3 dir = agent.target.variable - agent.transform.position;
            dir.Normalize();
            Ray ray = new Ray
            {
                origin = agent.transform.position - dir * agent.avoidanceRadius.variable,
                direction = dir
            };
            foreach (var hit in Physics.SphereCastAll(ray, agent.avoidanceRadius.variable, agent.maxSeeAhead.variable + agent.avoidanceRadius.variable))
            {
                if (hit.collider.gameObject != agent.gameObject)
                {
                    float forceStrength = hit.distance - agent.avoidanceRadius.variable;
                    forceStrength *= forceStrength;
                    forceStrength = Mathf.Max(forceStrength, 0.1f);
                    Vector3 force = (hit.point - hit.collider.bounds.center).normalized / forceStrength;
                    avoidanceForce += force;
                }
            }

            return avoidanceForce;
        }

#if UNITY_EDITOR
        public override void DrawGizmo(SteeringAgent agent)
        {
            Vector3 avoidanceForce = Vector3.zero;
            Vector3 dir = agent.target.variable - agent.transform.position;
            dir.Normalize();
            Ray ray = new Ray
            {
                origin = agent.transform.position - dir * agent.avoidanceRadius.variable,
                direction = dir
            };
            Gizmos.color = Color.blue;

            foreach (var hit in Physics.SphereCastAll(ray, agent.avoidanceRadius.variable, agent.avoidanceRadius.variable + agent.maxSeeAhead.variable))
            {
                if (hit.collider.gameObject != agent.gameObject)
                {
                    Vector3 force = (hit.point - hit.collider.bounds.center).normalized * (1.0f - hit.distance / agent.maxSeeAhead.variable);
                    Gizmos.DrawSphere(hit.point, 0.05f);
                    Gizmos.DrawLine(hit.point, hit.point + force);
                }
            }

            Gizmos.color = Color.green;
            Gizmos.DrawWireSphere(ray.origin, agent.avoidanceRadius.variable);
            Gizmos.DrawLine(agent.transform.position, agent.transform.position + avoidanceForce);
        }
#endif

    }

}