﻿using System.Collections.Generic;
using UnityEngine;

namespace hry.labs.NPC.AI.Steering
{

    public class SteeringAgent : Agent
    {

        public Rigidbody rigidBody;
        public Utility.FloatVariable moveForce;
        public Utility.FloatVariable maxSeeAhead;
        public Utility.Vector3Variable target;
        public Utility.Vector3Variable targetVelocity;
        public Utility.FloatVariable avoidanceRadius;
        public Utility.FloatVariable slowingRadius;
        public Utility.FloatVariable pursuitPrediction;
        public List<Steer> movement = new List<Steer>();
#if UNITY_EDITOR
        public bool drawGizmos = true;
#endif
        
        public Vector3 SteeringDirection
        {
            get;
            private set;
        }



        private void Start()
        {
            SteeringDirection = (target.variable - transform.position).normalized;
        }

        private void LateUpdate()
        {
            transform.LookAt(target.variable, Vector3.up);
        }

        private void FixedUpdate()
        {
            Vector3 force = Vector3.zero;
            Vector3 moveVelocity = Vector3.zero;

            foreach (var m in movement)
            {
                force += m.Act(this);
            }

            force = force.normalized * moveForce.variable;
            moveVelocity = force / rigidBody.mass;

            rigidBody.AddForce(moveVelocity, ForceMode.Impulse);

            SteeringDirection = rigidBody.velocity.sqrMagnitude < 0.0001f ? (target.variable - transform.position).normalized : rigidBody.velocity.normalized;
        }

#if UNITY_EDITOR
        private void OnDrawGizmos()
        {
            if (drawGizmos)
            {
                Gizmos.color = Color.black;
                Gizmos.DrawLine(transform.position, transform.position + SteeringDirection);

                Gizmos.color = Color.cyan;
                Gizmos.DrawWireSphere(target.variable, 0.15f);
                Gizmos.DrawLine(transform.position, transform.position + rigidBody.velocity);

                foreach (var m in movement)
                {
                    m.DrawGizmo(this);
                }
            }
        }
#endif
    }

}