﻿using UnityEngine;

namespace hry.labs.Player
{

    public class PlayerMovement : MonoBehaviour
    {

        public PlayerData playerData;
        public Utility.FloatVariable playerSpeed;
        public Utility.Vector3Variable moveVelocity;
        public Utility.Vector3Variable playerPosition;

        private Vector3 _moveVelocity = Vector3.zero;
        private CharacterController _playerCharacterController;


        private void Awake()
        {
            // Initialize shared variables
            playerSpeed.variable = 0;
            moveVelocity.variable = Vector3.zero;
            playerPosition.variable = transform.position;
        }

        void Start()
        {
            _playerCharacterController = GetComponent<CharacterController>();
        }

        void Update()
        {
            UpdateVelocity();
            UpdateRotation();
            UpdateSpeed();
        }

        private void LateUpdate()
        {
            playerPosition.variable = transform.position;
        }

        private void UpdateSpeed()
        {
            playerSpeed.variable = _moveVelocity.magnitude * Vector3.Dot(_moveVelocity, transform.forward);
        }

        private void UpdateRotation()
        {
            Vector3 dir = Input.mousePosition - UnityEngine.Camera.main.WorldToScreenPoint(transform.position);
            transform.rotation = Quaternion.LookRotation(new Vector3(dir.x, 0.0f, dir.y));
        }

        private void UpdateVelocity()
        {
            // Check if on character is on the ground and if so, add velocities
            if (_playerCharacterController.isGrounded)
            {
                // Get vertical and horizontal axis speed and jump to the _moveVelocity
                float forwardSpeed = Input.GetAxis("Vertical");
                float sidewaySpeed = Input.GetAxis("Horizontal");
                bool jump = Input.GetButtonDown("Jump");

                _moveVelocity = (transform.right * sidewaySpeed + transform.forward * forwardSpeed).normalized * playerData.movementSpeed;

                // If player is grounded and jumps, add y-axis speed
                if (jump)
                {
                    _moveVelocity.y = playerData.jumpSpeed;
                }
            }

            // Apply gravity acceleration
            _moveVelocity.y = _moveVelocity.y - 9.8f * Time.deltaTime;
            moveVelocity.variable = _moveVelocity;
            // Multiply speed by time to get absolute deltas
            _playerCharacterController.Move(_moveVelocity * Time.deltaTime);
        }
    }

}