﻿using UnityEngine;
using UnityEngine.UI;


namespace hry.labs.UI
{

    [RequireComponent(typeof(Combat.Damageable))]
    public class HealthBar : MonoBehaviour
    {

        public GameObject uiPrefab;

        private Transform _uiTransform;
        private Image _healthImage;
        private Utility.FloatVariable _HP;


        private void Start()
        {
            _uiTransform = Instantiate<GameObject>(uiPrefab).transform;

            Canvas[] canvases = FindObjectsOfType<Canvas>();
            // Find the canvas with world space render settings
            foreach (Canvas c in canvases)
            {
                if (c.renderMode == RenderMode.WorldSpace)
                {
                    _uiTransform.SetParent(c.transform, false);
                    break;
                }
            }

            _HP = GetComponent<Combat.Damageable>().HP;
            _healthImage = _uiTransform.GetComponentInChildren<Image>();
            UpdateHealth(1.0f);
        }

        public void UpdateHealth(float fraction)
        {
            _healthImage.fillAmount = fraction;
            _healthImage.color = Color.Lerp(Color.red, Color.green, fraction);
        }

        public void Update()
        {
            UpdateHealth(_HP.variable / _HP.defaultValue);
        }

        private void LateUpdate()
        {
            _uiTransform.position = transform.position;
        }

        private void OnDestroy()
        {
            if (_uiTransform != null)
                Destroy(_uiTransform.gameObject);
        }

    }

}