﻿using hry.labs.Control;
using System.IO;
using UnityEngine;
using UnityEngine.Events;
using UnityEngine.SceneManagement;
using UnityEngine.UI;

namespace hry.labs.UI.Menu
{

    public class Menu : MonoBehaviour
    {

        public Text playerNameText;
        public Text[] names;
        public Text[] scores;
        public GameState gameState;
        public Object nextScene;


        /// <summary>
        /// Checks if user did enter his name and if so, loads the game level
        /// with the user name and with score set to 0.
        /// </summary>
        public void StartGame()
        {
            if (!string.IsNullOrEmpty(playerNameText.text))
            {
                gameState.currentPlayer.variable = playerNameText.text;
                gameState.currentScore.variable = 0;
                SceneManager.LoadScene(nextScene.name);
            }
        }

        public void LoadGameState()
        {
            // Check if there is existing save, if not use default state
            if (File.Exists(Utility.Constants.PATH_TO_SAVE))
            {
                var saveJsonString = File.ReadAllText(Utility.Constants.PATH_TO_SAVE);
                gameState = ScriptableObject.CreateInstance<GameState>();
                JsonUtility.FromJsonOverwrite(saveJsonString, gameState);
            }
        }

        private void CleanMenu()
        {
            // Clean all texts and set them to saved state or default state
            for (int i = 0; i < names.Length; ++i)
            {
                names[i].text = "";
                scores[i].text = "";
            }

            for (int i = 0; i < gameState.lastPlayers.Count; ++i)
            {
                names[i].text = gameState.lastPlayers[i].ToString();
                scores[i].text = gameState.lastScores[i].ToString();
            }
        }

        private void Start()
        {
            LoadGameState();
            CleanMenu();
        }

    }

}

